/*
 * SNPFile -- SNP data management for association mapping.
 *
 * Copyright (C) 2007 by Bioinformatics Research Center,
 *                       University of Aarhus, Denmark
 *                       Contact: Thomas Mailund <mailund@birc.au.dk>
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#include "byte_order_handler.hh"
#include "archive_common.hh"



/** \file
 * \brief Implementations of WriteOrderHandler and ReadOrderHandler.
 */



namespace BiRC {
    namespace SNPFile {
        namespace {
            template<typename T> void
            primitiveWrite(WriteOrderHandler *h, T val)
            {
                char *pos = h->getPosition();
                val = toLittleEndian(val);
                memcpy(pos, &val, sizeof(T));
                h->setPosition(pos + sizeof(T));
            }
        }



        WriteOrderHandler &WriteOrderHandler::operator<<(uint32_t i)
        {
            primitiveWrite(this, i);
            return *this;
        }



        WriteOrderHandler &WriteOrderHandler::operator<<(int32_t i)
        {
            primitiveWrite(this, i);
            return *this;
        }



        WriteOrderHandler &WriteOrderHandler::operator<<(char i)
        {
            primitiveWrite(this, i);
            return *this;
        }



        WriteOrderHandler &WriteOrderHandler::operator<<(const char *str)
        {
            size_t len = strlen(str) + 1;
            memcpy(i_position, str, len);
            i_position += len;
            return *this;
        }



        namespace {
            template<typename T> void
            primitiveRead(ReadOrderHandler *h, T &t)
            {
                const char *pos = h->getPosition();
                memcpy(&t, pos, sizeof(T));
                t = fromLittleEndian(t);
                h->setPosition(pos + sizeof(T));
            }
        }



        ReadOrderHandler &ReadOrderHandler::operator>>(uint32_t &i)
        {
            primitiveRead(this, i);
            return *this;
        }



        ReadOrderHandler &ReadOrderHandler::operator>>(int32_t &i)
        {
            primitiveRead(this, i);
            return *this;
        }



        ReadOrderHandler &ReadOrderHandler::operator>>(char &i)
        {
            primitiveRead(this, i);
            return *this;
        }



        ReadOrderHandler &ReadOrderHandler::operator>>(char *dest)
        {
            size_t len = strlen(i_position) + 1;
            memcpy(dest, i_position, len);
            i_position += len;
            return *this;
        }
    }
}

