/*
 * SNPdata -- SNP data management for association mapping.
 *
 * Copyright (C) 2007 by Bioinformatics Research Center,
 *                       University of Aarhus, Denmark
 *                       Contact: Thomas Mailund <mailund@birc.au.dk>
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#include "snpfile.hh"
#include "matrix.hh"
#include "file_matrix.hh"
#include "metadata_access.hh"
using namespace BiRC::SNPFile;

#include <cstdlib>
#include <iostream>

static void
count_case_control(const ImmutableMatrix &matrix, 
		   const std::vector<bool> &status, 
		   int col)
{
    int a0 = 0, a1 = 0, a2 = 0,
	u0 = 0, u1 = 0, u2 = 0,
	missing_a = 0, missing_u = 0;

    for (int i = 0; i < matrix.noRows(); ++i)
	if (status[i] == 1)
	    // Affected individual...
	    if (matrix(i,col) == 0)      ++a0;
	    else if (matrix(i,col) == 1) ++a1;
	    else if (matrix(i,col) == 2) ++a2;
	    else ++missing_a;
	else
	    // Unaffected individual
	    if (matrix(i,col) == 0)      ++u0;
	    else if (matrix(i,col) == 1) ++u1;
	    else if (matrix(i,col) == 2) ++u2;
	    else ++missing_u;


    std::cout << col << ' '
	      << a0 << ' ' << a2 << ' ' << a1 << ' ' << missing_a << ' '
	      << u0 << ' ' << u2 << ' ' << u1 << ' ' << missing_u << std::endl;
}


int main(int argc, char *argv[])
{
    if (argc < 2)
	{
	    std::cerr << "Usage: " << argv[0] << " SNP-file [index]" << std::endl;
	    return EXIT_FAILURE;
	}

    try {
	SNPFile input(argv[1]);
	ImmutableMatrix matrix(newReadOnlyFileMatrix(input));

	std::vector<bool> status;
	getMetadata(input, "phenotype", status);

	std::cout << "index "
		  << "a.00 a.01 a.11 a.missing "
		  << "u.00 u.01 u.11 u.missing " << std::endl;
	if (argc > 2)
	    for (int i = 2; i < argc; ++i)
		{
		    int idx = atoi(argv[i]);
		    if (idx < 0 || idx > matrix.noCols())
			std::cerr << "Index " << idx << " out of bound." << std::endl;
		    count_case_control(matrix, status, idx);
		}
	else
	    for (int i = 0; i < matrix.noCols(); ++i)
		count_case_control(matrix, status, i);

    } catch (std::exception &e) {
	std::cerr << "Error: " << e.what() << std::endl;
	return EXIT_FAILURE;
    }

    return EXIT_SUCCESS;
}
