/*
 * SNPFile -- SNP data management for association mapping.
 *
 * Copyright (C) 2007 by Bioinformatics Research Center,
 *                       University of Aarhus, Denmark
 *                       Contact: Thomas Mailund <mailund@birc.au.dk>
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */



#undef NDEBUG			// enable asserts always

#include "array_matrix.hh"
#include "matrix.hh"
#include "iterators.hh"

#include <string>
#include <iostream>
#include <sstream>



using namespace BiRC::SNPFile;



int main()
{
    Matrix m = newArrayMatrix(3, 4);
    for(int i = 0; i < m.noCols(); i++)
    {
        for(int j = 0; j < m.noRows(); j++)
            m(j, i) = (j * m.noCols() + i + 1) % 10;
    }

    std::string expected;
    std::ostringstream found;
    MatrixView v = m.view();



    expected =
        "1 2 3 4 \n"
        "5 6 7 8 \n"
        "9 0 1 2 \n";
    found << v;
    assert(expected == found.str());
    found.str("");



    {
        expected =
            "1 2 \n"
            "5 6 \n"
            "9 0 \n"
            "2 3 \n"
            "6 7 \n"
            "0 1 \n"
            "3 4 \n"
            "7 8 \n"
            "1 2 \n";
        const ColumnWindowIterator begin(v, 2), end;
        for(ColumnWindowIterator it = begin; it != end; ++it)
            found << *it;
        assert(expected == found.str());
        found.str("");
    }



    {
        expected =
            "1 \n"
            "5 \n"
            "9 \n"
            "2 \n"
            "6 \n"
            "0 \n"
            "3 \n"
            "7 \n"
            "1 \n"
            "4 \n"
            "8 \n"
            "2 \n";
        const ColumnIterator begin(v), end;
        for(ColumnIterator it = begin; it != end; it += 1)
            found << *it;
        assert(expected == found.str());
        found.str("");
    }



    {
        expected =
            "1 2 3 4 \n"
            "5 6 7 8 \n"
            "5 6 7 8 \n"
            "9 0 1 2 \n";
        const RowWindowIterator begin(v, 2), end;
        for(RowWindowIterator it = begin; it != end; it = it + 1)
            found << *it;
        assert(expected == found.str());
        found.str("");
    }



    {
        expected =
            "1 2 3 4 \n"
            "5 6 7 8 \n"
            "9 0 1 2 \n";
        const RowIterator begin(v), end;
        for(RowIterator it = begin; it != end; it = 1 + it)
            found << *it;
        assert(expected == found.str());
        found.str("");
    }



    return 0;
}
