/*
 * SNPFile -- SNP data management for association mapping.
 *
 * Copyright (C) 2007 by Bioinformatics Research Center,
 *                       University of Aarhus, Denmark
 *                       Contact: Thomas Mailund <mailund@birc.au.dk>
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#undef NDEBUG			// enable asserts always

#include "matrix.hh"
#include "file_matrix.hh"
#include "byte_order_handler.hh"
#include "snpfile.hh"

#include <iostream>
#include <cassert>
#include <cstring>

#include <unistd.h>


using namespace BiRC::SNPFile;



int
main()
{
    try
    {
        const char *filename = "test_snpfile.snp";

        unlink(filename);

        SNPFile file(filename, true);


        //Low-level stuff:
        const allele_t matrixTestData[] = {1, 2, 3, 5, 7, 42, 14};
        const size_t matrixTestDataLength = sizeof(matrixTestData) / sizeof(allele_t);

        const size_t nRepeats = 4;

        file.resizeMatrix(matrixTestDataLength, nRepeats);
        for(size_t i = 0; i < nRepeats; i++)
            memcpy(file.accessMatrixData() + matrixTestDataLength*i,
                   matrixTestData,
                   sizeof(matrixTestData));

        const std::string key0 = "Key 0";
        const std::string key1 = "Key 1";
        const std::string key2 = "Key 2";
        const char metadata0[] = "Hello world"; // 12 bytes.
        const char metadata1[] = "foo"; // 4 bytes.
        const char metadata2[] = "bar"; // 4 bytes.

        file.getMetadataBuffer(key0).reset(metadata0, sizeof(metadata0));
        file.getMetadataBuffer(key1).reset(metadata1, sizeof(metadata1));
        file.getMetadataBuffer(key2).reset(metadata2, sizeof(metadata2));



        //Matrix stuff:
        Matrix matrix(newReadWriteFileMatrix(file));

        for(size_t i = 0; i < matrixTestDataLength; i++)
            matrix(i, matrix.noCols() - 1) = allele_t(40 + i);

        matrix.hideCol(2);
        matrix.hideRow(3);
        matrix.hideRow(4);
        matrix.storeHiding();

        file.close();
    }
    catch(std::runtime_error &ex)
    {
        std::cout << ex.what() << std::endl;
        assert(false);
    }

    return 0;
}
