/*
 * SNPFile -- SNP data management for association mapping.
 *
 * Copyright (C) 2007 by Bioinformatics Research Center,
 *                       University of Aarhus, Denmark
 *                       Contact: Thomas Mailund <mailund@birc.au.dk>
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#undef NDEBUG
#include "snpfile.hh"
#include "matrix.hh"
#include "file_matrix.hh"
#include "timing.hh"

#include <fstream>

#include <cstdlib>



using namespace BiRC::SNPFile;



namespace
{
    std::string filename = "/tmp/benchmark_test.snp";
}



/** \file
 * \brief Simple test of access time on matrices.
 */



TimeInterval writeMatrixByColumn(int rows, int cols)
{
    TimePoint start;

    SNPFile snpFile(filename, true);
    Matrix m = newReadWriteFileMatrix(snpFile);

    m.resize(0, 0);
    std::cout << "Testing " << rows << "x" << cols << " matrix." << std::endl;

    m.resize(rows, cols);
    int f1 = 0, f2 = 1;
    //Fill with fibonacci numbers.
    for(int i = 0; i < cols; i++)
    {
        for(int j = 0; j < rows; j++)
        {
            int f = f1 + f2;
            m(j, i) = f;
            f1 = f2;
            f2 = f;
        }
    }

    snpFile.close();

    TimePoint end;

    TimeInterval result = end - start;
    std::cout << result << std::endl;
    return result;
}



TimeInterval writeMatrixByRow(int rows, int cols)
{
    TimePoint start;

    SNPFile snpFile(filename, true);
    Matrix m = newReadWriteFileMatrix(snpFile);

    m.resize(0, 0);
    std::cout << "Testing " << rows << "x" << cols << " matrix." << std::endl;

    m.resize(rows, cols);
    int f1 = 0, f2 = 1;
    //Fill with fibonacci numbers.
    for(int i = 0; i < rows; i++)
    {
        for(int j = 0; j < cols; j++)
        {
            int f = f1 + f2;
            m(i, j) = f;
            f1 = f2;
            f2 = f;
        }
    }

    snpFile.close();

    TimePoint end;

    TimeInterval result = end - start;
    std::cout << result << std::endl;
    return result;
}



TimeInterval benchmarkMatrix(std::string accessTypeArg, int rows, int cols)
{
    if(accessTypeArg == "row")
        return writeMatrixByRow(rows, cols);
    else
        return writeMatrixByColumn(rows, cols);
}



int main(int argc, char** argv)
{
    if(argc != 5)
    {
        std::cout << "Usage " << argv[0] << " resultFile (row|col) nuRows nuCols" << std::endl;
        return 1;
    }

    std::ofstream resultFile(argv[1], std::ios::app);

    std::string accessTypeArg(argv[2]);
    if(accessTypeArg != "row" && accessTypeArg != "col")
    {
        std::cout << "Illegal access pattern argument" << std::endl;
        return 1;
    }

    int rows = atoi(argv[3]);
    int cols = atoi(argv[4]);

    TimeInterval result;
    
    result = benchmarkMatrix(accessTypeArg, rows, cols);

    resultFile << accessTypeArg                 << " "
               << rows                          << " "
               << cols                          << " "
               << (size_t(rows) * size_t(cols)) << " "
               << result.getUserTime()          << " "
               << result.getNiceTime()          << " "
               << result.getSystemTime()        << " "
               << result.getIdleTime()          << " "
               << result.getIOWaitTime()        << " "
               << result.getHarwareIRQTime()    << " "
               << result.getSoftwareIRQTime()   << std::endl;


    resultFile.close();

    return 0;
}
