/*
 * SNPFile -- SNP data management for association mapping.
 *
 * Copyright (C) 2007 by Bioinformatics Research Center,
 *                       University of Aarhus, Denmark
 *                       Contact: Thomas Mailund <mailund@birc.au.dk>
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#include "metadata.hh"



/** \file
 * \brief Implementations of functions of basic metadata classes.
 */



namespace BiRC {
    namespace SNPFile {
        MetadataBuffer::MetadataBuffer(size_t size)
        {
            i_size = size;
            i_data = new char[size];
        }



        MetadataBuffer::MetadataBuffer(const char *data, size_t size)
        {
            i_size = size;
            i_data = new char[size];
            memcpy(i_data, data, size);
        }



        MetadataBuffer::~MetadataBuffer()
        {
            delete[] i_data;
        }



        void MetadataBuffer::resize(size_t size)
        {
            delete[] i_data;
            i_size = size;
            i_data = new char[size];
        }


        void MetadataBuffer::reset(const char *data, size_t size)
        {
            resize(size);
            memcpy(i_data, data, size);
        }



        MetadataAccessor::~MetadataAccessor()
        {
        }
    }
}
