/*
 * SNPFile -- SNP data management for association mapping.
 *
 * Copyright (C) 2007 by Bioinformatics Research Center,
 *                       University of Aarhus, Denmark
 *                       Contact: Thomas Mailund <mailund@birc.au.dk>
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#undef NDEBUG   //Always enable assertions



#include <cassert>

#include "metadata_access.hh"
#include "snpfile.hh"
#include "test_util.hh"

#include <vector>
#include <map>
#include <set>
#include <string>

#include <iostream>



using namespace BiRC::SNPFile;



int main()
{
    int fd = 0;
    char *fn = makeTempFile(fd);
    SNPFile file(fn, true);


    int i = 31337;
    char c = 'h';
    std::vector<int> vi;
    for(int j = 0; j < 65535; j++)
        vi.push_back(j);
    std::vector<bool> vb;
    for(int j = 0; j < 300; j++)
        vb.push_back(j % 3 == 1);
    std::map<std::string, unsigned> mm;
    mm["foo"] = 7;
    mm["bar"] = 9;
    mm["baz"] = 13;

    const std::string key0 = "Key 0";
    const std::string key1 = "Key 1";
    const std::string key2 = "Key 2";
    const std::string key3 = "Key 3";
    const std::string key4 = "Key 4";
    const std::string keyX = "Key X";

    setMetadata(file, key0, i);
    setMetadata(file, key1, c);
    setMetadata(file, key2, vi);
    setMetadata(file, key3, vb);
    setMetadata(file, key4, mm);

    assert(containsMetadata(file, key0));
    assert(!containsMetadata(file, keyX));

    assert(isTypeName<int>(metadataType(file, key0)));
    assert(isTypeName<char>(metadataType(file, key1)));


    int readI;
    char readC;
    std::vector<int> readVi;
    std::vector<bool> readVb;
    std::map<std::string, unsigned> readMm;

    readI = fetchMetadata<int>(file, key0);
    readC = fetchMetadata<char>(file, key1);
    getMetadata(file, key2, readVi);
    getMetadata(file, key3, readVb);
    getMetadata(file, key4, readMm);

    assert(i == readI);
    assert(c == readC);
    assert(vi == readVi);
    assert(vb == readVb);
    assert(mm == readMm);

    removeMetadata(file, key2);
    assert(!containsMetadata(file, key2));

    std::set<std::string> keySet;
    keySet.insert(key0);
    keySet.insert(key1);
    keySet.insert(key3);
    keySet.insert(key4);

    MetadataKeyIterator begin = metadataKeysBegin(file), end = metadataKeysEnd(file);
    for(MetadataKeyIterator it = begin; it != end; ++it)
    {
        std::set<std::string>::iterator found = keySet.find(*it);
        assert(found != keySet.end());
        keySet.erase(found);
    }
    assert(keySet.size() == 0);

    removeTempFile(fd, fn);


    return 0;
}
