/*
 * SNPFile -- SNP data management for association mapping.
 *
 * Copyright (C) 2006 by Thomas Mailund <mailund@mailund.dk>
 *                    and Søren Besenbacher <besen@birc.au.dk>
 *
 * Copyright (C) 2007 by Bioinformatics Research Center,
 *                       University of Aarhus, Denmark
 *                       Contact: Thomas Mailund <mailund@birc.au.dk>
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */



#undef NDEBUG			// enable asserts always

#include "metadata_filtering.hh"
#include "array_matrix.hh"

#include <cassert>

#include <vector>
#include <string>
#include <set>
#include <map>



using namespace BiRC::SNPFile;



int main()
{
    Matrix m(newArrayMatrix());
    m.hideRow(1);
    m.hideRow(2);
    m.hideRow(0);

    m.hideCol(7);
    m.hideCol(13);
    m.hideCol(9);



    std::string sourceStr = "Hello World!";
    std::string resultStr;

    resultStr = filterOnIndexInHiddenCols(m, sourceStr);
    assert(resultStr == "Hello Wrd!");
    resultStr.clear();

    filterOnIndexInHiddenCols(m, sourceStr, resultStr);
    assert(resultStr == "Hello Wrd!");
    resultStr.clear();

    resultStr = filterOnIndexInHiddenRows(m, sourceStr);
    assert(resultStr == "lo World!");
    resultStr.clear();

    filterOnIndexInHiddenRows(m, sourceStr, resultStr);
    assert(resultStr == "lo World!");
    resultStr.clear();



    std::vector<int>  sourceVec;
    sourceVec.push_back(1);
    sourceVec.push_back(9);
    sourceVec.push_back(0);
    std::set<int> resultSet;
    std::vector<int> resultVec;

    resultVec = filterOnContentsInHiddenCols(m, sourceVec);
    assert(resultVec.size() == 2);
    assert(resultVec[0] == 1);
    assert(resultVec[1] == 0);
    resultVec.clear();

    filterOnContentsInHiddenCols(m, sourceVec, resultSet);
    assert(resultSet.size() == 2);
    assert(resultSet.count(0) == 1);
    assert(resultSet.count(1) == 1);
    resultSet.clear();

    resultVec = filterOnContentsInHiddenRows(m, sourceVec);
    assert(resultVec.size() == 1);
    assert(resultVec[0] == 9);
    resultVec.clear();

    filterOnContentsInHiddenRows(m, sourceVec, resultSet);
    assert(resultSet.size() == 1);
    assert(resultSet.count(9) == 1);
    resultSet.clear();



    std::map<int, std::string> sourceMap;
    std::map<int, std::string> resultMap;

    sourceMap[0] = "Hello";
    sourceMap[1] = "World";
    sourceMap[13] = "42";

    resultMap = filterOnKeysInHiddenCols(m, sourceMap);
    assert(resultMap.size() == 2);
    assert(resultMap[0] == "Hello");
    assert(resultMap[1] == "World");
    resultMap.clear();

    filterOnKeysInHiddenCols(m, sourceMap, resultMap);
    assert(resultMap.size() == 2);
    assert(resultMap[0] == "Hello");
    assert(resultMap[1] == "World");
    resultMap.clear();

    resultMap = filterOnKeysInHiddenRows(m, sourceMap);
    assert(resultMap.size() == 1);
    assert(resultMap[13] == "42");
    resultMap.clear();

    filterOnKeysInHiddenRows(m, sourceMap, resultMap);
    assert(resultMap.size() == 1);
    assert(resultMap[13] == "42");
    resultMap.clear();
}
