/*
 * SNPdata -- SNP data management for association mapping.
 *
 * Copyright (C) 2007 by Bioinformatics Research Center,
 *                       University of Aarhus, Denmark
 *                       Contact: Thomas Mailund <mailund@birc.au.dk>
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#include "snpfile.hh"
#include "metadata_access.hh"
using namespace BiRC::SNPFile;

#include <cstdlib>
#include <iostream>

template <typename PhenoType>
static void printPhenotype(SNPFile &input, std::ostream &outstream)
{
    std::vector<PhenoType> status;
    getMetadata(input, "phenotype", status);
    std::ostream_iterator<PhenoType> out(outstream, " ");
    std::copy(status.begin(), status.end(), out);
    outstream << std::endl;
}

int main(int argc, char *argv[])
{
    if (argc != 2)
	{
	    std::cerr << "Usage: " << argv[0] << " SNP-file" << std::endl;
	    return EXIT_FAILURE;
	}

    try {
	SNPFile input(argv[1]);

	if (fetchMetadata<bool>(input, "case/control"))
	    printPhenotype<bool>(input, std::cout);
	else if (fetchMetadata<bool>(input, "quantitative"))
	    printPhenotype<double>(input, std::cout);
	else
	    {
		std::cerr << "Cannot find case/control nor quantitative "
			  << "metadata in this file."
			  << std::endl;
		return EXIT_FAILURE;
	    }

    } catch (std::exception &e) {
	std::cerr << "Error: " << e.what() << std::endl;
	return EXIT_FAILURE;
    }

    return EXIT_SUCCESS;
}
