/*
 * SNPFile -- SNP data management for association mapping.
 *
 * Copyright (C) 2008 by Bioinformatics Research Center,
 *                       University of Aarhus, Denmark
 *                       Contact: Thomas Mailund <mailund@birc.au.dk>
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */
#ifndef TIMING_HH_INCLUDED
#define TIMING_HH_INCLUDED



#include <iostream>



namespace BiRC
{
    namespace SNPFile
    {
        class TimeInterval;



        class TimePoint
        {
            friend class TimeInterval;

            int user, nice, system, idle, io, irq, softIrq;

        public:
            TimePoint();
        };



        TimeInterval operator- (const TimePoint &to, const TimePoint &from);



        class TimeInterval
        {
            int dUser, dNice, dSystem, dIdle, dIo, dIrq, dSoftIrq;

        public:
            TimeInterval()
                : dUser(0), dNice(0), dSystem(0), dIdle(0), dIrq(0), dSoftIrq(0)
            {}

            TimeInterval(const TimePoint &from, const TimePoint &to);

            double getUserTime() const
            {
                return dUser / 100.0;
            }

            double getNiceTime() const
            {
                return dNice / 100.0;
            }

            double getSystemTime() const
            {
                return dSystem / 100.0;
            }

            double getIdleTime() const
            {
                return dIdle / 100.0;
            }

            double getIOWaitTime() const
            {
                return dIo / 100.0;
            }

            double getHarwareIRQTime() const
            {
                return dIrq / 100.0;
            }

            double getSoftwareIRQTime() const
            {
                return dSoftIrq / 100.0;
            }
        };



        std::ostream &operator<< (std::ostream &out, const TimeInterval &ti);
    }
}



#endif
